/**
 * dos2unix '\n' convertor 0.5.0
 *   based on Unix2Dos 0.9.0 by Peter Hanecak (made 19.2.1997)
 * Copyright 1997,.. by Peter Hanecak <hanecak@megaloman.com>.
 * All rights reserved.
 *
 * dos2unix filters reading input from stdin and writing output to stdout.
 * Without arguments it reverts the format (e.i. if source is in UNIX format,
 * output is in DOS format and vice versa).
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * See the COPYING file for license information.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <getopt.h>

#define PARAM_CT_UNIX2DOS	"u2d"
#define PARAM_CT_DOS2UNIX	"d2u"
#define PARAM_CT_AUTO		"auto"

#define CT_AUTO	0
#define CT_UNIX2DOS	1
#define CT_DOS2UNIX	2

void help();
int convert(char *fn, int ConvType);

int main(int argc, char *argv[]) {
	int ConvType = CT_AUTO;
	int o;

	static struct option long_options[] = {
		{ PARAM_CT_AUTO, no_argument, NULL, 'A' },
		{ PARAM_CT_UNIX2DOS, no_argument, NULL, 'D' },
		{ PARAM_CT_DOS2UNIX, no_argument, NULL, 'U' },
		{ "help", no_argument, NULL, 'h' },
		{ 0, 0, 0, 0 }
	};

	// process parameters
	while ((o = getopt_long(argc, argv, "ADUh", long_options, NULL)) != EOF) {
		switch (o) {
		case 'A':
			ConvType = CT_AUTO;
			break;
		case 'D':
			ConvType = CT_UNIX2DOS;
			break;
		case 'U':
			ConvType = CT_DOS2UNIX;
			break;
		case 'h':
			help();
			return 0;
		case '?':
			help();
			return -1;
		}
	}

	if (optind < argc) {
		while(optind < argc)
			if ((o = convert(argv[optind++], ConvType)) < 0)
				break;
	}
	else
		o = convert(NULL, ConvType);

	return o;
}

// if fn is NULL then input is stdin and output is stdout
int convert(char *fn, int ConvType) {
	char c;
	char *tempFn;
	FILE *in = stdin, *out = stdout;

	if (fn != NULL) {
		if ((in = fopen(fn, "r")) == NULL) {
			perror(fn);
			return -1;
		}
		tempFn = tmpnam(NULL);
		if ((out = fopen(tempFn, "w")) == NULL) {
			perror("dos2unix");
			return -2;
		}
	}

	while ((c = fgetc(in)) != EOF) {
		if (c == '\r') {
			if ((ConvType == CT_UNIX2DOS) && (fn != NULL)) {
				// file is alredy in DOS format so it is not necessery to touch it
				if (fclose(in) < 0) {
					perror("dos2unix");
					return -2;
				}
				if (fclose(out) < 0) {
					perror("dos2unix");
					return -2;
				}
				if (remove(tempFn) < 0) {
					perror("dos2unix");
					return -2;
				}
				return 0;
			}
			if (!ConvType)
				ConvType = CT_DOS2UNIX;
			break;
		}
		if (c == '\n') {
			if ((ConvType == CT_DOS2UNIX) && (fn != NULL)) {
				// file is alredy in UNIX format so it is not necessery to touch it
				if (fclose(in) < 0) {
					perror("dos2unix");
					return -2;
				}
				if (fclose(out) < 0) {
					perror("dos2unix");
					return -2;
				}
				if (remove(tempFn) < 0) {
					perror("dos2unix");
					return -2;
				}
				return 0;
			}
			if (!ConvType)
				ConvType = CT_UNIX2DOS;
			if (ConvType == CT_UNIX2DOS)
				fputc('\r', out);
			fputc('\n', out);
			break;
		}
		fputc(c, out);
	}
	if (c != EOF)
		while ((c = fgetc(in)) != EOF) {
			if (c == '\r')
				continue;
			if (c == '\n') {
				if (ConvType == CT_UNIX2DOS)
					fputc('\r', out);
				fputc('\n', out);
				continue;
			}
		fputc(c, out);
	}

	if (fn != NULL) {
		if (fclose(in) < 0) {
			perror("dos2unix");
			return -2;
		}
		if (fclose(out) < 0) {
			perror("dos2unix");
			return -2;
		}

		if ((in = fopen(tempFn, "r")) == NULL) {
			perror("dos2unix");
			return -1;
		}
		if ((out = fopen(fn, "w")) == NULL) {
			perror(fn);
			return -2;
		}

		while ((c = fgetc(in)) != EOF)
			fputc(c, out);

		if (fclose(in) < 0) {
			perror("dos2unix");
			return -2;
		}
		if (fclose(out) < 0) {
			perror("dos2unix");
			return -2;
		}

		if (remove(tempFn) < 0) {
			perror("dos2unix");
			return -2;
		}
	}

	return 0;
}

void help() {
	printf("usage:\tdos2unix [--d2u|--u2d|--auto|-D|-U|-A] [<file name> [...]]\n"
		"\t\t--d2u, -U\toutput will be in UNIX format\n"
		"\t\t--u2d, -D\toutput will be in DOS format\n"
		"\t\t--auto, -A\toutput will be set based upon autodetection of source format\n"
		"- when no options are given then input format will be automaticaly detected\n"
		"  and converted to another format on output\n"
		"- when no file is given, then stdin is used as input and\n"
		"  stdout as output\n");
}
